pub struct Microseconds(pub u64);Expand description
Timestamp in microseconds since Unix epoch.
Used for journal entry timestamps and fine-grained time operations.
Tuple Fields
0: u64Implementations
Sourceimpl Microseconds
impl Microseconds
Sourcepub fn new(microseconds: u64) -> Microseconds
pub fn new(microseconds: u64) -> Microseconds
Create a timestamp from microseconds.
Sourcepub fn now() -> Microseconds
pub fn now() -> Microseconds
Get the current time as microseconds since Unix epoch.
Sourcepub fn to_seconds(self) -> Seconds
pub fn to_seconds(self) -> Seconds
Convert to seconds (truncates).
Sourcepub fn saturating_add(self, other: Microseconds) -> Microseconds
pub fn saturating_add(self, other: Microseconds) -> Microseconds
Add two durations with saturation at the numeric bounds.
Sourcepub fn saturating_sub(self, other: Microseconds) -> Microseconds
pub fn saturating_sub(self, other: Microseconds) -> Microseconds
Subtract two durations with saturation at the numeric bounds.
Sourcepub fn checked_add(self, other: Microseconds) -> Option<Microseconds>
pub fn checked_add(self, other: Microseconds) -> Option<Microseconds>
Checked addition. Returns None if overflow occurred.
Sourcepub fn checked_sub(self, other: Microseconds) -> Option<Microseconds>
pub fn checked_sub(self, other: Microseconds) -> Option<Microseconds>
Checked subtraction. Returns None if overflow occurred.
Sourcepub fn is_multiple_of(self, other: Microseconds) -> bool
pub fn is_multiple_of(self, other: Microseconds) -> bool
Returns true if this duration is a multiple of the other duration.
Useful for checking if bucket durations align.